@@ -32,43 +32,44 @@ class Debug:
     Whitelist = [
         "About_Dialog",
         "Assert_Dialog",
-        "BiotSavart_JIT",
         "BiotSavart_CUDA",
+        "BiotSavart_JIT",
         "CalculationThread",
         # "Config",
         "Constants",
         "Constraint",
-        # "Constraint_Editor",
-        # "Display_Widget",
-        # "Field",
-        # "Field_Widget",
+        "Constraint_Editor",
+        "Display_Widget",
+        "Field",
+        "Field_Widget",
         "Groupbox",
         "GUI",
         "HLine",
         "IconLabel",
-        # "Menu",
-        # "Metric",
+        "Menu",
+        "Metric",
         "Metric_Presets",
-        # "Metric_Widget",
+        "Metric_Widget",
         "Model",
-        # "ModelAccess",
-        # "Parameters",
-        # "Parameters_Widget",
+        "ModelAccess",
+        "Parameters",
+        "Parameters_Widget",
         "Perspective_Presets",
-        # "Perspective_Widget",
-        # "SamplingVolume",
-        # "SamplingVolume_Widget",
+        "Perspective_Widget",
+        "SamplingVolume",
+        "SamplingVolume_Widget",
         "SidebarLeft",
         "SidebarRight",
         "SliderFloat",
-        # "Statusbar",
+        "Statusbar",
         # "Table",
+        "Theme",
         "Usage_Dialog",
         "Version",
         # "VispyCanvas",
-        # "Wire",
+        "Wire",
         "Wire_Presets",
-        # "Wire_Widget"
+        "Wire_Widget"
     ]
 
     def __init__(self, obj, text: str, color=None, force: bool = False):
